# Description:
#   Make sure that hubot shares the go coding-style.
#
# Dependencies:
#   None
#
# Configuration:
#   None
#
# Commands:
#   hubot the go style - Make hubot explain how to write practical go.
#
# Author:
#   mrinjamul
#
# Notes:
#   These rules were provided by :
#   https://github.com/cristaloleg/go-advice
#

goRules = [
  "0. Always `go fmt` your code"
  "1.  Each package fulfils a single purpose",
  "2.  Handle errors explicitly",
  "3.  Return early rather than nesting deeply",
  "4.  Leave concurrency to the caller",
  "5.  Before you launch a goroutine, know when it will stop",
  "6.  Avoid package level state"
  "7.  Simplicity matters"
  "8.  Write tests to lock in the behaviour of your package’s API"
  "9.  If you think it’s slow, first prove it with a benchmark"
  "10. Moderation is a virtue"
  "11. Maintainability counts"
  "12. Don't communicate by sharing memory, share memory by communicating"
  "13. Concurrency is not parallelism"
  "14. Channels orchestrate; mutexes serialize."
  "15. The bigger the interface, the weaker the abstraction"
  "16. Make the zero value useful"
  "17. `interface{}` says nothing"
  "18. Gofmt's style is no one's favorite, yet gofmt is everyone's favorite"
  "19. A little copying is better than a little dependency"
  "20. Syscall must always be guarded with build tags"
  "21. Cgo must always be guarded with build tags"
  "22. Cgo is not Go"
  "23. With the unsafe package there are no guarantees"
  "24. Clear is better than clever"
  "25. Reflection is never clear"
  "26. Errors are values"
  "27. Don't just check errors, handle them gracefully"
  "28. Design the architecture, name the components, document the details"
  "29. Documentation is for users"
  "30.  Don't panic"
]

module.exports = (robot) ->
  robot.respond /(what are )?the go (rules|laws|style)/i, (msg) ->
    msg.send goRules.join('\n')
